// SPDX-License-Identifier: MIT
pragma solidity ^0.8.13;

contract DutchAuction {

    struct AuctionItem {
        string name;
        uint startPrice;
        uint endPrice;
        uint duration;
        uint interval;
        uint priceDecrement;
        address buyer;
    }

    struct AvailableItem {
        string name;
        uint price;
        uint timestamp;
        string status;
    }

    struct currentItem{
        string name;
        uint startPrice;
        uint endPrice;
        uint duration;
        uint interval;
        uint priceDecrement;
        uint timestamp;
        string status;
        uint currentPrice;
    }

    uint public startTimestamp;
    AuctionItem[] public items;
    AvailableItem[] public AvailableItems;
    currentItem public current;
    uint public currentItemIndex;
    bool public ended = false;

    address public vendor;
    bool public recovered = false;


    constructor(string[] memory _names, uint _startTimestamp, uint[] memory _startPrices, uint[] memory _endPrices, uint[] memory _durations, uint[] memory _intervals) {
        require(_startPrices.length == _endPrices.length && _endPrices.length == _durations.length && _durations.length == _intervals.length, "Invalid input lengths");

        startTimestamp = _startTimestamp;
        vendor = msg.sender;

        for (uint i = 0; i < _startPrices.length; i++) {
            require(_startPrices[i] > 0, "Start price must be greater than 0");
            require(_endPrices[i] < _startPrices[i], "End price must be less than start price");
            require(_durations[i] > 0, "Duration must be greater than 0");
            require(_intervals[i] > 0 && _intervals[i] <= _durations[i], "Invalid interval");

            AuctionItem memory newItem = AuctionItem({
                name: _names[i],
                startPrice: _startPrices[i],
                endPrice: _endPrices[i],
                duration: _durations[i],
                interval: _intervals[i],
                priceDecrement: (_startPrices[i] - _endPrices[i]) / (_durations[i] / _intervals[i]),
                buyer: address(0)
            });

            items.push(newItem);
        }

        currentItemIndex = 0;
    }

    function createAuction(
        string memory _name,
        uint _startPrice,
        uint _endPrice,
        uint _duration,
        uint _interval
    ) public {
        require(msg.sender == vendor, "Only vendor can create auction");
        require(_startPrice > 0, "Start price must be greater than 0");
        require(_endPrice < _startPrice, "End price must be less than start price");
        require(_duration > 0, "Duration must be greater than 0");
        require(_interval > 0 && _interval <= _duration, "Invalid interval");

        items.push(AuctionItem({
            name: _name,
            startPrice: _startPrice,
            endPrice: _endPrice,
            duration: _duration,
            interval: _interval,
            priceDecrement: (_startPrice - _endPrice) / (_duration / _interval),
            buyer: address(0)
        }));
        if(ended){
            startTimestamp = block.timestamp;
            ended = false;
        }
    }


    function getCurrentPrice() public returns (uint) {
        AuctionItem storage ThisCurrentItem = items[currentItemIndex];
        uint currentTimestamp = startTimestamp;

        require(ThisCurrentItem.buyer == address(0), "Item already sold");
        require(block.timestamp >= currentTimestamp, "Auction not started");
        while (block.timestamp >= currentTimestamp + ThisCurrentItem.duration) {
            if (currentItemIndex < items.length - 1) {
                currentItemIndex++;
                currentTimestamp = currentTimestamp + ThisCurrentItem.duration;
                ThisCurrentItem = items[currentItemIndex];
            } else {
                ended = true;
                break;
            }
        }
        require(!ended, "Auction ended");
        uint timePassed = block.timestamp - currentTimestamp;
        uint price = ThisCurrentItem.startPrice - ThisCurrentItem.priceDecrement * (timePassed / ThisCurrentItem.interval);

        //return (price > ThisCurrentItem.endPrice) ? price : ThisCurrentItem.endPrice;
        return price;
    }

    function getCurrentIndex() public returns (uint) {
        AuctionItem storage ThisCurrentItem = items[currentItemIndex];

        require(ThisCurrentItem.buyer == address(0), "Item already sold");
        require(block.timestamp >= startTimestamp, "Auction not started");
        while (block.timestamp >= startTimestamp + ThisCurrentItem.duration) {
            if (currentItemIndex < items.length - 1) {
                currentItemIndex++;
            } else {
                ended = true;
                break;
            }
        }
        require(!ended, "Auction ended");
        return currentItemIndex;
    }

    function buy() external payable {
        require(currentItemIndex < items.length, "All items already sold");
        require(!ended, "Auction ended");
        uint price = getCurrentPrice();

        require(msg.value == price, "Invalid price");
        payable(msg.sender).transfer(msg.value);
        AuctionItem storage ThisCurrentItem = items[currentItemIndex];
        ThisCurrentItem.buyer = msg.sender;

        // Passer à l'article suivant s'il y en a un
        if (currentItemIndex < items.length - 1) {
            currentItemIndex++;
            startTimestamp = block.timestamp;
        } else {
            ended = true;
        }
    }

    function recoverFunds() public {
        require(msg.sender == vendor, "Only vendor can recover ether");
        require(ended, "Auction not ended");
        require(!recovered, "Ether already recovered");
        payable(msg.sender).transfer(address(this).balance);
        recovered = true;
    }

    function isEnded() public view returns (bool){
        return ended;
    }

    function isRecovered() public view returns (bool){
        return recovered;
    }

    function getAllAuctionItems() public returns (AvailableItem[] memory) {
        uint currentIndex = getCurrentIndex();
        uint oldItemDuration = 0;

        delete AvailableItems;

        for (uint i = currentIndex; i < items.length; i++) {
            string memory itemName;
            uint itemPrice;
            uint itemTimestamp;
            string memory itemStatus;

            if (i == currentIndex) {
                itemPrice = getCurrentPrice();
                itemTimestamp = startTimestamp;
                itemStatus = "En Cours";
            } else {
                itemPrice = items[i].startPrice;
                itemTimestamp = startTimestamp + oldItemDuration + items[i - 1].duration;
                oldItemDuration += items[i - 1].duration;
                itemStatus = "A Venir";
            }
            itemName = items[i].name;
            AvailableItems.push(AvailableItem({
                name: itemName,
                price: itemPrice,
                timestamp: itemTimestamp,
                status: itemStatus
            }));
        }
        return AvailableItems;
    }

    function getAuctionItem(uint index) public returns (currentItem memory) {
        delete current;
        current.name = items[index].name;
        current.startPrice = items[index].startPrice;
        current.endPrice = items[index].endPrice;
        current.duration = items[index].duration;
        current.interval = items[index].interval;
        current.priceDecrement = items[index].priceDecrement;
        current.timestamp = startTimestamp;
        if (index == currentItemIndex){
            current.status = "En Cours";
            current.currentPrice = getCurrentPrice();
        } else if (index < currentItemIndex){
            current.status = "Vendu";
            current.currentPrice = items[index].endPrice;
        } else {
            current.status = "A Venir";
            current.currentPrice = items[index].startPrice;
        }
        return current;
    }

    function getItems() public view returns (AuctionItem[] memory) {
        return items;
    }

    function getStartTimestamp() public view returns (uint){
        return startTimestamp;
    }
}