"use client";

import React, { useEffect, useState } from 'react';
import { useRouter, useParams } from 'next/navigation';
import { Typography, Card, CardContent, Divider, Button, Box } from '@mui/material';
import { format } from 'date-fns';
import { fr } from 'date-fns/locale';
import Web3 from 'web3';

import NavBar from '../../components/NavBar';
import { checkConnectionMetaMask, getAuctionContract } from '../../utils';

const AuctionDetail = () => {
    const { auctionContract, web3 } = getAuctionContract();
    const router = useRouter();
    const params = useParams();
    const id = params.id;
    const [auctionDetails, setAuctionDetails] = useState(null);

    useEffect(() => {
        const checkConnection = async () => {
            const accounts = await checkConnectionMetaMask();
            if (accounts.length === 0) {
                router.push('/');
            } else {
                try {
                    const auction = await auctionContract.methods.getAuctionItem(id - 1).call();

                    const date = new Date(Number(auction.timestamp) * 1000);
                    const formattedTimestamp = date.toLocaleString('fr-FR');

                    setAuctionDetails({
                        itemName: auction.name,
                        itemStartPrice: auction.startPrice,
                        itemEndPrice: auction.endPrice,
                        itemDuration: auction.duration,
                        itemInterval: auction.interval,
                        itemPriceDecrement: auction.priceDecrement,
                        itemTimestamp: formattedTimestamp,
                        itemStatus: auction.status,
                        itemCurrentPrice: auction.currentPrice
                    });
                } catch (error) {
                    console.error('Error fetching current item:', error);
                }
            }
        };
        checkConnection();
    }, []);

    if (!auctionDetails) {
        return null;
    }

    const handleReturnAllList = () => {
        router.push(`/auction-list`);
    };

    const handleBid = async () => {
        try {
            await auctionContract.methods.buy().send({
                from: window.ethereum.selectedAddress,
                value: web3.utils.toWei(auctionDetails.itemCurrentPrice, 'ether'),
                gas: 30000,
            });
            console.log("Auction successful!");
            router.push(`/auction-list`);
        } catch (error) {
            console.error("Error during auction:", error.message);
        }
    };

    return (
        <>
            <NavBar />
            <Box m={2}>
                <Card>
                    <CardContent>
                        <Typography variant="h5">Détails de l'Enchère</Typography>
                        <Divider />
                        <Typography variant="subtitle1">
                            Nom: {auctionDetails.itemName}
                            <br />
                            Prix de départ: {web3.utils.fromWei(auctionDetails.itemStartPrice.toString(), 'ether')} Ether
                            <br />
                            Prix final: {web3.utils.fromWei(auctionDetails.itemEndPrice.toString(), 'ether')} Ether
                            <br />
                            Durée: {parseInt(auctionDetails.itemDuration) / 60 } minutes
                            <br />
                            Intervalle: {parseInt(auctionDetails.itemInterval) / 60} minutes
                            <br />
                            Valeur de décrément: {web3.utils.fromWei(auctionDetails.itemPriceDecrement.toString(), 'ether')} Ether
                            <br />
                            Timestamp: {auctionDetails.itemTimestamp}
                            <br />
                            Statut: {auctionDetails.itemStatus}
                            <br />
                            Prix actuel: {web3.utils.fromWei(auctionDetails.itemCurrentPrice.toString(), 'ether')} Ether
                        </Typography>
                        <Box mt={1}>
                            <Button onClick={handleReturnAllList} variant="contained" color="primary" sx={{ marginRight: '8px' }}>
                                Retour à la liste
                            </Button>

                            {auctionDetails.itemStatus === 'En Cours' && (
                                <>
                                    <Button onClick={handleBid} variant="contained" color="secondary">
                                        Enchérir avec le prix actuel
                                    </Button>
                                </>
                            )}
                        </Box>
                    </CardContent>
                </Card>
            </Box>
        </>
    );
};

export default AuctionDetail;
