"use client";

import React, { useEffect, useState } from 'react';
import Web3 from 'web3';
import { Container, Typography, Card, CardContent, List, ListItem, Paper, Grid, Button, Divider, Box } from '@mui/material';
import { useRouter } from 'next/navigation';
import Link from 'next/link';

import NavBar from '../components/NavBar';
import { checkConnectionMetaMask, getAuctionContract } from '../utils';

const AuctionList = () => {
    const [auctions, setAuctions] = useState([]);
    const { auctionContract, web3 } = getAuctionContract();
    const router = useRouter();

    useEffect(() => {
        const fetchItemsAvailable = async () => {
            const accounts = await checkConnectionMetaMask();
            if (accounts.length === 0) {
                router.push('/');
            } else {
                try {
                    const availableAuctions = await auctionContract.methods.getAllAuctionItems().call();
                    console.log('Available Auctions:', availableAuctions);

                    setAuctions(availableAuctions);
                } catch (error) {
                    console.error('Error fetching available auctions:', error);
                    setAuctions([]);
                }
            }
        };

        fetchItemsAvailable();
    }, []);

    const formatTimestamp = (timestamp) => {
        const date = new Date(Number(timestamp) * 1000);
        const formattedDate = date.toLocaleString('fr-FR');
        return formattedDate;
    };

    const handleViewDetails = (auctionId) => {
        router.push(`/auction-list/${auctionId+1}`);
    };

    const handleBid = async (auctionId) => {
        try {
            await auctionContract.methods.buy().send({
                from: window.ethereum.selectedAddress,
                value: web3.utils.toWei(auctions[auctionId].price, 'ether'),
                gas: 30000,
            });

            console.log("Auction successful!");
        } catch (error) {
            console.error("Error during auction:", error.message);
        }
    };

    return (
        <>
            <NavBar />
            <Container>
                <Typography variant="h2">Liste des Enchères Disponibles</Typography>
                <List>
                    {auctions.map((auction, index) => (
                        <React.Fragment key={index}>
                            <ListItem>
                                <Typography variant="subtitle1">
                                    Nom: {auction.name}
                                    <br />
                                    Prix: {web3.utils.fromWei(auction.price.toString(), 'ether')} Ether
                                    <br />
                                    Timestamp: {formatTimestamp(auction.timestamp)}
                                    <br />
                                    Statut: {auction.status}
                                </Typography>
                            </ListItem>
                            <ListItem>
                                <Button onClick={() => handleViewDetails(index)} variant="contained" color="primary">
                                    Voir détail
                                </Button>
                                {auction.status === 'En Cours' && (
                                    <>
                                        <Box m={1} />
                                        <Button onClick={() => handleBid(index)} variant="contained" color="secondary">
                                            Enchérir avec le prix actuel
                                        </Button>
                                    </>
                                )}
                            </ListItem>
                            <Divider />
                        </React.Fragment>
                    ))}
                </List>
            </Container>
        </>
    );
};

export default AuctionList;
